"""
 * @file   : materialVideo.py
 * @time   : 15:23
 * @date   : 2024/3/23
 * @mail   : 9727005@qq.com
 * @creator: ShanDong Xiedali
 * @company: HiLand & RainyTop
"""
from JianYingDraft.core.media import Media
from JianYingDraft.core import template
from JianYingDraft.utils import tools


class MediaVideo(Media):

    def _set_material_data_for_content(self):
        """
        设置草稿文件的content部分
        """
        speed_id = tools.generate_id()
        scm_id = tools.generate_id()
        canvas_id = tools.generate_id()

        self.material_data_for_content["speeds"] = template.get_speed(speed_id)
        self.material_data_for_content["sound_channel_mappings"] = template.get_sound_channel_mapping(scm_id)
        self.material_data_for_content["canvases"] = template.get_canvas(canvas_id)

        video_data = self.__gen_video()

        # 判定是否要对视频素材本身的背景音进行静音处理
        bgm_mute = self.kwargs.get("bgm_mute", False)
        if bgm_mute:
            video_data["extra_type_option"] = 1
        else:
            video_data["extra_type_option"] = 0
        pass

        self.material_data_for_content["videos"] = video_data

        # 将素材的各种业务信息，暂时保存起来，后续供track下的segment使用
        self.material_data_for_content["X.extra_material_refs"] = [speed_id, scm_id, canvas_id]

    def __gen_video(self):
        entity = template.get_video(self.id)
        entity["duration"] = self.duration
        entity["height"] = self.height
        entity["local_material_id"] = self.id  # 暂时跟素材设置为相同的id
        entity["material_name"] = self.material_name
        entity["path"] = self.file_Path
        entity["type"] = self.material_type
        entity["width"] = self.width
        return entity


pass



"""
 * @file   : materialVideo.py
 * @time   : 15:23
 * @date   : 2024/3/23
 * @mail   : 9727005@qq.com
 * @creator: ShanDong Xiedali
 * @company: HiLand & RainyTop
"""
import time
from JianYingDraft.utils import tools


def get_canvas(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "album_image": "",
        "blur": 0.0,
        "color": "",
        "id": guid,
        "image": "",
        "image_id": "",
        "image_name": "",
        "source_platform": 0,
        "team_id": "",
        "type": "canvas_color"
    }


def get_sound_channel_mapping(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "audio_channel_mapping": 0,
        "id": guid,
        "is_config_open": False,
        "type": "none"
    }


def get_speed(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "curve_speed": None,
        "id": guid,
        "mode": 0,
        "speed": 1.0,
        "type": "speed"
    }


def get_material_for_meta_info(guid: str = None):
    """
    获取素材信息
    :param guid: 素材id
    :return:
    """
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "create_time": int(time.time()),
        "duration": 0,
        "extra_info": "",
        "file_Path": "",
        "height": 0,
        "id": guid,
        "import_time": int(time.time()),
        "import_time_ms": int(time.time()) * 10 ^ 6,
        "md5": "",
        "metetype": "",  # meta or mate?? 估计剪映开发人员最初拼写错误，以后大家就以讹传讹将错就错了。
        "roughcut_time_range": {"duration": 0, "start": 0},
        "sub_time_range": {"duration": -1, "start": -1},
        "type": 0,
        "width": 0
    }


def get_track(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "attribute": 0,  # 0表示正常；1表示关闭本轨道
        "flag": 0,
        "id": guid,
        "segments": [],
        "type": ""
    }


def get_segment(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "cartoon": False,
        "clip": {
            "alpha": 1.0,
            "flip": {"horizontal": False, "vertical": False},
            "rotation": 0.0,
            "scale": {"x": 1.0, "y": 1.0},
            "transform": {"x": 0.0, "y": 0.0}
        },
        "common_keyframes": [],
        "enable_adjust": True,
        "enable_color_curves": True,
        "enable_color_wheels": True,
        "enable_lut": True,
        "enable_smart_color_adjust": False,
        "extra_material_refs": [
        ],
        "group_id": "",
        "hdr_settings": {"intensity": 1.0, "mode": 1, "nits": 1000},
        "id": guid,
        "intensifies_audio": False,
        "is_placeholder": False,
        "is_tone_modify": False,
        "keyframe_refs": [],
        "last_nonzero_volume": 1.0,
        "material_id": "",
        "render_index": 0,
        "reverse": False,
        "source_timerange": {"duration": 0, "start": 0},
        "speed": 1.0,
        "target_timerange": {"duration": 0, "start": 0},
        "template_id": "",
        "template_scene": "default",
        "track_attribute": 0,
        "track_render_index": 0,
        "visible": True,
        "volume": 1.0
    }


def get_beat(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "ai_beats": {
            "beats_path": "",
            "beats_url": "",
            "melody_path": "",
            "melody_percents": [0.0],
            "melody_url": ""
        },
        "enable_ai_beats": False,
        "gear": 404,
        "id": guid,
        "mode": 404,
        "type": "beats",
        "user_beats": [],
        "user_delete_ai_beats": None
    }


def get_video(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "audio_fade": None,
        "cartoon_path": "",
        "category_id": "",
        "category_name": "local",
        "check_flag": 63487,
        "crop": {
            "lower_left_x": 0.0,
            "lower_left_y": 1.0,
            "lower_right_x": 1.0,
            "lower_right_y": 1.0,
            "upper_left_x": 0.0,
            "upper_left_y": 0.0,
            "upper_right_x": 1.0,
            "upper_right_y": 0.0
        },
        "crop_ratio": "free",
        "crop_scale": 1.0,
        "duration": 0,
        "extra_type_option": 0,  # 是否播放视频素材的本身的背景音。0：播放；1：不播放
        "formula_id": "",
        "freeze": None,
        "gameplay": None,
        "has_audio": True,
        "height": 0,
        "id": guid,
        "intensifies_audio_path": "",
        "intensifies_path": "",
        "is_unified_beauty_mode": False,
        "local_id": "",
        "local_material_id": "",
        "material_id": "",
        "material_name": "",
        "material_url": "",
        "matting": {
            "flag": 0,
            "has_use_quick_brush": False,
            "has_use_quick_eraser": False,
            "interactiveTime": [],
            "path": "",
            "strokes": []
        },
        "media_path": "",
        "object_locked": None,
        "path": "",
        "picture_from": "none",
        "picture_set_category_id": "",
        "picture_set_category_name": "",
        "request_id": "",
        "reverse_intensifies_path": "",
        "reverse_path": "",
        "source_platform": 0,
        "stable": None,
        "team_id": "",
        "type": "",
        "video_algorithm": {
            "algorithms": [],
            "deflicker": None,
            "motion_blur_config": None,
            "noise_reduction": None,
            "path": "",
            "time_range": None
        },
        "width": 0
    }


def get_audio(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "app_id": 0,
        "category_id": "",
        "category_name": "local",
        "check_flag": 1,
        "duration": 0,
        "effect_id": "",
        "formula_id": "",
        "id": guid,
        "intensifies_path": "",
        "local_material_id": "",
        "music_id": tools.generate_id(),
        "name": "Krubb Wenkroist - Bleach.mp3",
        "path": "D:/Music/Krubb Wenkroist - Bleach.mp3",
        "request_id": "",
        "resource_id": "",
        "source_platform": 0,
        "team_id": "",
        "text_id": "",
        "tone_category_id": "",
        "tone_category_name": "",
        "tone_effect_id": "",
        "tone_effect_name": "",
        "tone_speaker": "",
        "tone_type": "",
        "type": "extract_music",
        "video_id": "",
        "wave_points": []
    }


def get_text(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "add_type": 0,
        "alignment": 1,
        "background_alpha": 1.0,
        "background_color": "",
        "background_height": 1.0,
        "background_horizontal_offset": 0.0,
        "background_round_radius": 0.0,
        "background_style": 0,
        "background_vertical_offset": 0.0,
        "background_width": 1.0,
        "bold_width": 0.0,
        "border_color": "",
        "border_width": 0.08,
        "check_flag": 7,
        "content": "<font id=\"\" path=\"E:/JianyingPro/4.2.0.10100/Resources/Font/SystemFont/zh-hans.ttf\"><color=(1.000000, 1.000000, 1.000000, 1.000000)><size=15.000000>[默认文本]</size></color></font>",
        "font_category_id": "",
        "font_category_name": "",
        "font_id": "",
        "font_name": "",
        "font_path": "E:/JianyingPro/4.2.0.10100/Resources/Font/SystemFont/zh-hans.ttf",
        "font_resource_id": "",
        "font_size": 15.0,
        "font_source_platform": 0,
        "font_team_id": "",
        "font_title": "none",
        "font_url": "",
        "fonts": [],
        "force_apply_line_max_width": False,
        "global_alpha": 1.0,
        "group_id": "",
        "has_shadow": False,
        "id": guid,
        "initial_scale": 1.0,
        "is_rich_text": False,
        "italic_degree": 0,
        "ktv_color": "",
        "language": "",
        "layer_weight": 1,
        "letter_spacing": 0.0,
        "line_spacing": 0.02,
        "name": "",
        "preset_category": "",
        "preset_category_id": "",
        "preset_has_set_alignment": False,
        "preset_id": "",
        "preset_index": 0,
        "preset_name": "",
        "recognize_type": 0,
        "relevance_segment": [],
        "shadow_alpha": 0.8,
        "shadow_angle": -45.0,
        "shadow_color": "",
        "shadow_distance": 8.0,
        "shadow_point": {"x": 1.0182337649086284, "y": -1.0182337649086284},
        "shadow_smoothing": 1.0,
        "shape_clip_x": False,
        "shape_clip_y": False,
        "style_name": "",
        "sub_type": 0,
        "text_alpha": 1.0,
        "text_color": "#FFFFFF",
        "text_preset_resource_id": "",
        "text_size": 30,
        "text_to_audio_ids": [],
        "tts_auto_update": False,
        "type": "text",
        "typesetting": 0,
        "underline": False,
        "underline_offset": 0.22,
        "underline_width": 0.05,
        "use_effect_default_color": True,
        "words": []
    }


def get_material_animation(guid: str = None):
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "animations": [],
        "id": guid,
        "type": "sticker_animation"
    }


def get_audio_fade(guid: str = None, fade_in_duration: int = 0, fade_out_duration: int = 0):
    """
    添加音频的淡入淡出效果
    @param guid:
    @param fade_in_duration: 淡入时间（单位微秒）
    @param fade_out_duration: 淡出时间（单位微秒）
    @return:
    """
    if guid is None:
        guid = tools.generate_id()
    pass

    return {
        "fade_in_duration": fade_in_duration,
        "fade_out_duration": fade_out_duration,
        "fade_type": 0,
        "id": guid,
        "type": "audio_fade"
    }



import os
import shutil
import uuid
import json

from BasicLibrary.io.dirHelper import DirHelper


def generate_id() -> str:
    """
    生成uuid
    """
    return str(uuid.uuid4()).upper()


def read_json(path):
    """
    读取json文件
    :param path: 文件路径
    """
    with open(path, 'r') as file:
        return json.load(file)
    pass


def write_json(path, data):
    """
    写入json文件
    :param path: 文件路径
    :param data: 数据
    """
    with open(path, 'w') as file:
        # 给json.dump添加参数 ensure_ascii=false可以保证汉字不被编码
        json.dump(data, file)
    pass


def create_folder(folder_path):
    """
    创建文件夹
    :param folder_path: 文件夹路径
    """
    if os.path.exists(folder_path):
        shutil.rmtree(folder_path)
    pass

    DirHelper.ensure_exist(folder_path)



"""
 * @file   : materialFactory.py
 * @time   : 16:10
 * @date   : 2024/3/24
 * @mail   : 9727005@qq.com
 * @creator: ShanDong Xiedali
 * @company: HiLand & RainyTop
"""
import os

from BasicLibrary.data.stringHelper import StringHelper
from BasicLibrary.environment.dynamicImporter import DynamicImporter
from pymediainfo import MediaInfo


class MediaFactory:
    """
    媒体工厂
    """

    @staticmethod
    def create(media_full_name: str, **kwargs):
        """
        根据素材来信创建素材实体
        :param media_full_name:
        :return:
        """
        if not os.path.exists(media_full_name):
            return None
        pass

        media_info = MediaInfo.parse(media_full_name).to_data()["tracks"][1]
        material_type = media_info['track_type'].lower()

        material_type = StringHelper.upper_first_char(material_type)
        package_name = f"JianYingDraft.core.media{material_type}"
        class_name = f"Media{material_type}"

        kwargs["mediaInfo"] = media_info
        kwargs["mediaFileFullName"] = media_full_name

        material = DynamicImporter.load_class(package_name, class_name, **kwargs)

        return material



"""
 * @file   : __.py
 * @time   : 11:12
 * @date   : 2024/3/20
 * @mail   : 9727005@qq.com
 * @creator: ShanDong Xiedali
 * @company: HiLand & RainyTop
"""

import os
import time

from BasicLibrary.configHelper import ConfigHelper
from BasicLibrary.data.dateTimeHelper import DateTimeHelper

from JianYingDraft.utils import tools
from JianYingDraft.core.media import Media
from JianYingDraft.core.mediaFactory import MediaFactory
from JianYingDraft.core import template


class Draft:
    _draft_content_file_base_name = "draft_content.json"
    _draft_meta_info_file_base_name = "draft_meta_info.json"

    def __init__(self, name: str = ""):
        if not name:
            name = time.strftime("%Y%m%d.%H%M%S", time.localtime())
        pass

        # 草稿保存位置
        self._drafts_root = ConfigHelper.get_item("JianYingDraft.basic", "drafts_root", "C:\\Jianying.Drafts")
        self._draft_folder = os.path.join(self._drafts_root, name)

        # 从模板获取草稿的基础数据
        here = os.path.abspath(os.path.dirname(__file__))
        template_folder = os.path.join(os.path.dirname(here), "template")
        self._draft_content_data = tools.read_json(os.path.join(template_folder, self._draft_content_file_base_name))
        self._draft_meta_info_data = tools.read_json(
            os.path.join(template_folder, self._draft_meta_info_file_base_name))

        # 初始化草稿内容信息
        self._draft_content_data['id'] = tools.generate_id()

        # 初始化草稿元数据信息
        self._draft_meta_info_data['id'] = tools.generate_id()
        self._draft_meta_info_data['draft_fold_path'] = self._draft_folder.replace("\\", '/')
        self._draft_meta_info_data['draft_timeline_metetyperials_size_'] = 0
        self._draft_meta_info_data['tm_draft_create'] = DateTimeHelper.get_timestamp(formatter=16)
        self._draft_meta_info_data['tm_draft_modified'] = DateTimeHelper.get_timestamp(formatter=16)
        self._draft_meta_info_data['draft_root_path'] = self._drafts_root.replace("/", "\\")
        self._draft_meta_info_data['draft_removable_storage_device'] = self._drafts_root.split(':/')[0]

        # 为方便调用目标文件中的material部分，定义快捷变量
        self._materials_in_draft_content: {} = self._draft_content_data['materials']  # 草稿内容库的素材

        self._materials_in_draft_meta_info: {} = self._draft_meta_info_data['draft_materials']  # 草稿元数据库的素材
        self._videos_material_in_draft_meta_info = self._materials_in_draft_meta_info[0]['value']
        self._audios_material_in_draft_meta_info = self._materials_in_draft_meta_info[6]['value']  # type为8的那条

        self._tracks_in_draft_content: [] = self._draft_content_data['tracks']  # 草稿内容库的轨道

    def add_media(self, media_file_full_name: str, start=0, duration=0, index=0, **kwargs):
        """
        添加媒体到草稿
        """
        _index = index

        media = MediaFactory.create(media_file_full_name, start=start, duration=duration, **kwargs)

        if media is None:
            return
        pass

        # 将媒体信息添加到draft的素材库
        self.__add_media_to_content_materials(media)

        # 将媒体信息添加到draft的轨道库
        self.__add_media_to_content_tracks(media)

        # 将媒体信息添加到draft的元数据库
        self.__add_media_to_meta_info(media)

    def add_effect(self, effect_path: str, effect_name: str, start=0, duration=0, index=0):
        """
        添加特效到草稿
        @param index:
        @param duration:
        @param start:
        @param effect_path: 特效所在的路径（内置特效时请设置本参数为空；外置的特效时使用。）
        @param effect_name: 特效的名称（内置特效总共有限的几个名称中的一个；外置特效的名称随意设定。）
        """
        pass

    def calc_draft_duration(self):
        """
        获取（通过计算）草稿的时长
        """
        return self.__get_track_duration("video")

    def save(self):
        """
        保存草稿
        """
        # 校准时长信息
        self.__calc_duration()

        # 新建项目文件夹
        tools.create_folder(self._draft_folder)

        # 持久化草稿
        draft_content_file_full_name = os.path.join(self._draft_folder, self._draft_content_file_base_name)
        draft_meta_info_file_full_name = os.path.join(self._draft_folder, self._draft_meta_info_file_base_name)
        tools.write_json(draft_content_file_full_name, self._draft_content_data)
        tools.write_json(draft_meta_info_file_full_name, self._draft_meta_info_data)

    def __add_media_to_content_materials(self, media: Media):
        """
        添加媒体信息到素材内容库的素材部分：
        """

        for _key, _value in media.material_data_for_content.items():
            _key = str(_key)

            # 排除中转使用的临时信息
            if _key.startswith("X."):
                continue
            pass

            self._materials_in_draft_content[_key].append(_value)
        pass

    def __add_media_to_content_tracks(self, media: Media):
        """
        添加媒体信息到素材内容库的轨道部分：
        """

        all_tracks = self._tracks_in_draft_content
        target_track = None
        for _track in all_tracks:
            if _track["type"] == media.category_type:
                target_track = _track
                break
            pass
        pass

        if target_track is None:
            target_track = template.get_track()
            target_track["type"] = media.category_type
            self._tracks_in_draft_content.append(target_track)
        pass

        # 添加新片段之前轨道总时长
        track_duration = self.__get_track_duration(media.category_type)

        # 设置新segment的在轨道上的开始时间
        segment_source_timerange = media.segment_data_for_content["target_timerange"]
        segment_source_timerange["start"] = track_duration
        target_track["segments"].append(media.segment_data_for_content)

    def __add_media_to_meta_info(self, media: Media):
        """
        添加媒体信息到元数据库：
        """

        if media.category_type == "video":
            self._videos_material_in_draft_meta_info.append(media.data_for_meta_info)
        else:
            self._audios_material_in_draft_meta_info.append(media.data_for_meta_info)
        pass

    def __calc_duration(self):
        """
        计算并设置草稿的总时长
        计算策略为（以视频时长为基准，让其他地方的duration相对此时长对齐）：
        1. 以视频轨道中最后一个segment的结束时间作为总时长
        2. 将音频轨道的总时长设置为第一步计算的结果
        3. 设置草稿的总时长字段duration（文件content和meta_info都要设置）
        """
        # 1. 获取视频轨道的总时长
        video_durations = self.__get_track_duration("video")
        # 2. 设置音频轨道的总时长
        # TODO:xiedali@2024/03/23 暂时只设置audio
        self.__set_track_duration("audio", video_durations)

        # 3. 设置草稿的总时长
        self._draft_content_data['duration'] = video_durations
        self._draft_meta_info_data['tm_duration'] = video_durations

    def __get_track_duration(self, track_type: str):
        all_tracks = self._tracks_in_draft_content
        target_track = None
        for _track in all_tracks:
            if _track["type"] == track_type:
                target_track = _track
                break
            pass
        pass

        if not target_track:
            return 0
        pass

        if len(target_track['segments']) == 0:
            return 0
        pass

        # 轨道总时长
        last_segment = target_track['segments'][-1]
        last_segment_timerange = last_segment['target_timerange']
        track_duration = last_segment_timerange['start'] + last_segment_timerange['duration']
        return track_duration

    def __set_track_duration(self, track_type: str, duration: int):
        all_tracks = self._tracks_in_draft_content
        target_track = None
        for _track in all_tracks:
            if _track["type"] == track_type:
                target_track = _track
                break
            pass
        pass

        if not target_track:
            return
        pass

        segments = target_track['segments']

        done = False
        for segment in segments:
            ss_timerange = segment['source_timerange']
            st_timerange = segment['target_timerange']

            if done:
                st_timerange['start'] = 0
                st_timerange['duration'] = 0

                ss_timerange['start'] = 0
                ss_timerange['duration'] = 0
            else:
                segment_start = st_timerange['start']
                segment_duration = st_timerange['duration']
                segment_end = segment_start + segment_duration

                if segment_end > duration:
                    ss_timerange['duration'] = duration - segment_start
                    st_timerange['duration'] = duration - segment_start
                    done = True
                pass
            pass
        pass



"""
 * @file   : materialVideo.py
 * @time   : 15:23
 * @date   : 2024/3/23
 * @mail   : 9727005@qq.com
 * @creator: ShanDong Xiedali
 * @company: HiLand & RainyTop
"""
from JianYingDraft.core import template
from JianYingDraft.core.media import Media
from JianYingDraft.utils import tools


class MediaAudio(Media):

    def _set_material_data_for_content(self):
        speed_id = tools.generate_id()
        scm_id = tools.generate_id()
        beat_id = tools.generate_id()

        self.material_data_for_content['speeds'] = template.get_speed(speed_id)
        self.material_data_for_content['sound_channel_mappings'] = template.get_sound_channel_mapping(scm_id)
        self.material_data_for_content['beats'] = template.get_beat(beat_id)

        self.material_data_for_content["audios"] = self.__gen_audio()
        # 将素材的各种业务信息，暂时保存起来，后续供track下的segment使用
        self.material_data_for_content["X.extra_material_refs"] = [speed_id, scm_id, beat_id]

        # 以下为音频淡入淡出的效果
        fade_in_duration = self.kwargs.get("fade_in_duration", 0)
        fade_out_duration = self.kwargs.get("fade_out_duration", 0)
        if fade_in_duration > 0 or fade_out_duration > 0:
            audio_fade_id = tools.generate_id()
            self.material_data_for_content['audio_fades'] = template.get_audio_fade(
                audio_fade_id,
                fade_in_duration,
                fade_out_duration
            )

            self.material_data_for_content["X.extra_material_refs"].append(audio_fade_id)
        pass

    def __gen_audio(self):
        entity = template.get_audio(self.id)
        entity["duration"] = self.duration
        entity["local_material_id"] = self.id
        entity["name"] = self.material_name
        entity["path"] = self.file_Path
        entity["type"] = "extract_" + self.material_type  # "extract_"??什么时候不加这个前缀
        return entity


pass
